"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("./helpers");
const db_1 = require("./helpers/db");
const insertPermissions_1 = require("./helpers/db/insertPermissions");
const getPublicPermissionsByModel_1 = require("./helpers/db/getPublicPermissionsByModel");
async function setPublicContentTypes({ strapi, actions = {}, plugins = {}, maxParallelOperations = 8, verbose = false, }) {
    function log(...args) {
        if (verbose) {
            strapi.log.info(...args);
        }
    }
    function warn(...args) {
        strapi.log.warn(...args);
    }
    if ((0, helpers_1.isEmpty)({ ...actions, ...plugins })) {
        warn(`No actions found in public-permissions plugin config.`);
        return;
    }
    log(`Setting actions to "public"...`);
    const configuredActions = Object.entries((0, helpers_1.replaceObjectKeyWithApiId)((0, helpers_1.replaceWildcardWithModelNames)(strapi, actions)));
    const configuredPlugins = Object.entries((0, helpers_1.replaceObjectKeyWithPluginId)(plugins));
    const { toDelete, toInsert } = (0, helpers_1.createDbOperationsLists)([
        ...configuredActions,
        ...configuredPlugins,
    ]);
    await strapi.db.connection.transaction(async function (trx) {
        const publicRole = await db_1.db.getPublicRole(trx);
        const chunkSize = maxParallelOperations;
        const chunks = [];
        for (let i = 0; i < toDelete.length; i += chunkSize) {
            chunks.push(toDelete.slice(i, i + chunkSize));
        }
        let idsToDelete = [];
        for (const chunk of chunks) {
            const permissions = await Promise.all(chunk.map((model) => (0, getPublicPermissionsByModel_1.getPublicPermissionsByModel)(trx, model)));
            const flattenedPermissions = permissions.flat();
            const arrayOfIds = flattenedPermissions.map(({ id }) => id);
            idsToDelete.push(...arrayOfIds);
        }
        log(`Deleting ${idsToDelete.length} permissions from table "${db_1.db.TABLE.permissions}"...`);
        await Promise.all([
            trx(db_1.db.TABLE.permissions).whereIn("id", idsToDelete).del(),
            await trx(db_1.db.TABLE.links).whereIn("permission_id", idsToDelete).del(),
        ]);
        log(`Adding ${toInsert.length} permissions to table "${db_1.db.TABLE.permissions}"...`);
        if (toInsert.length) {
            const now = new Date();
            await trx(db_1.db.TABLE.permissions).insert(toInsert.map((action) => ({
                action,
                created_at: now,
                updated_at: now,
            })));
        }
        const insertedIds = await (0, insertPermissions_1.insertPermissions)(trx, toInsert);
        log(`Adding ${insertedIds.length} links to table "${db_1.db.TABLE.links}"...`);
        if (insertedIds.length) {
            await trx(db_1.db.TABLE.links).insert(insertedIds.map(({ id }) => ({
                permission_id: id,
                role_id: publicRole.id,
            })));
        }
        log(`Finished setting actions to "public".`);
    });
}
exports.default = ({ strapi }) => {
    const plugin = strapi.plugin("public-permissions");
    setPublicContentTypes({
        strapi,
        actions: plugin.config("actions"),
        plugins: plugin.config("plugins"),
        maxParallelOperations: plugin.config("maxParallelOperations"),
        verbose: plugin.config("verbose"),
    });
};
